#version 330
#define SM_SIZE 4096.0
#define MAX_POWERUPS 10
//uncomment to get smoother shadows
//#define HQ_SHADOWS

uniform bool uni_HasDiffuseMap;
uniform bool uni_HasProjectiveTexture;

uniform vec3 uni_DiffuseColor;
uniform vec3 uni_AmbientColor;
uniform vec3 uni_SpecularColor;

uniform sampler2D uni_ProjTextureMap;
uniform sampler2D uni_DiffuseMap;
uniform sampler2DShadow uni_ShadowMap;
uniform vec3 uni_ParallelLightDirection;  //already in VIEW space!
uniform vec3 uni_ParallelLightColor;
uniform float uni_Shininess;
uniform int uni_ProjectorCount;

in vec2 var_TexCoord;
in vec3 var_Normal;
in vec3 var_EyeVec;
in vec4 var_SMTexCoords;
in vec4 var_TexCoordProj[MAX_POWERUPS];

out vec4 frag_Color;

const vec2 texmapscale = vec2(1.0/SM_SIZE,1.0/SM_SIZE);

float offset_lookup(const in sampler2DShadow map,
                    const in vec4 loc,
                    const in vec2 offset)
{
return textureProj(map, vec4(loc.xy + offset * texmapscale * loc.w,
                               loc.z, loc.w));
}

void main(void)
{
	vec4 baseColor = vec4(1,1,1,1);
	if(uni_HasDiffuseMap)
		baseColor = texture(uni_DiffuseMap,var_TexCoord);

	//simple alpha testing
	if(baseColor.a<0.1)
		discard;

	//do advanced PCF for shadows by using 16/4 samples (http://http.developer.nvidia.com/GPUGems/gpugems_ch11.html)
	float sum = 0;
	float x, y;

	#ifdef HQ_SHADOWS
	for (y = -1.5; y <= 1.5; y += 1.0)
	for (x = -1.5; x <= 1.5; x += 1.0)
		sum += offset_lookup(uni_ShadowMap,var_SMTexCoords,vec2(x,y));

	float shadowCoeff = sum/16.0;
	#else
	for (y = -0.5; y <= 0.5; y += 1.0)
	for (x = -0.5; x <= 0.5; x += 1.0)
		sum += offset_lookup(uni_ShadowMap,var_SMTexCoords,vec2(x,y));

	float shadowCoeff = sum/4.0;
	#endif	

	vec3 normal = normalize(var_Normal);

	//diffuse light = DiffMaterial * DiffLight * Lambert
	float diff = max(dot(normal,-uni_ParallelLightDirection),0.0);
	//float diff = max(dot(normal,-var_LightDir),0.0);
	vec3 diffuse = diff * uni_ParallelLightColor * uni_DiffuseColor;
	
	//ambient light = ambientMaterial * ambientLight
	vec3 ambient = uni_AmbientColor * 0.2;

	//specular light = specMaterial * specLight * phong
	float spec = 0.0;
	if(diff>0.0) //only calc if light shines on this point
	{
		vec3 R = reflect(normalize(uni_ParallelLightDirection),normal);		
		vec3 E = normalize(var_EyeVec);

		spec = pow(max(dot(normalize(R),E),0.0),uni_Shininess);
	}
	vec3 specular = spec * uni_SpecularColor * 0.8;

	vec4 sumProj = vec4(0,0,0,0);

	if(uni_HasProjectiveTexture)
	{
		int stop = uni_ProjectorCount;
		vec4 ProjMapColor[MAX_POWERUPS];

		for(int i=0;i<stop;i++) 
		{
			ProjMapColor[i] = texture2DProj(uni_ProjTextureMap, var_TexCoordProj[i]);
			if (var_TexCoordProj[i].w < 0 || var_TexCoordProj[i].x < 0 || var_TexCoordProj[i].y < 0 || var_TexCoordProj[i].x > var_TexCoordProj[i].w || var_TexCoordProj[i].y > var_TexCoordProj[i].w)
				ProjMapColor[i] = vec4(0,0,0,0);
			sumProj+=ProjMapColor[i];
		}
	}

	//final color = ambient + diffuse + specular
	frag_Color = vec4(ambient * baseColor.xyz + diffuse * baseColor.xyz * shadowCoeff + specular * shadowCoeff + sumProj.xyz, baseColor.a);
}